// AppleShooter.java

import ch.aplu.jgamegrid.*;
import ch.aplu.tcp.*;
import ch.aplu.util.*;
import java.util.*;
import javax.swing.JOptionPane;

public class AppleShooter extends GameGrid
{
  // ----------- Inner class MyTcpAgentAdapter --------------
  private class MyTcpAgentAdapter extends TcpAgentAdapter
  {
    public void dataReceived(String source, int[] data)
    {
      switch (data[0])
      {
        case Command.GAME_START:
          isLeft = source.equals(nickname);
          String msg = "Cursor up/down to target. <space> to shoot.";
          if (isLeft)
          {
            setTitle("LeftPlayer. " + msg);
            myName = nickname;
            teammate = nickname + "(1)";
            addKeyListener(leftCrossbow);
          }
          else
          {
            setTitle("RightPlayer. " + msg);
            myName = nickname + "(1)";
            teammate = nickname;
            addKeyListener(rightCrossbow);
          }
          setStatusText("Game started. You have " + nbBolt
            + " bolts in your quiver.");
          TcpTools.delay(2000);
          initialize();
          isOver = false;
          break;
        case Command.TEAMMATE_QUIT:
          initialize();
          setTitle(isLeft ? "LeftPlayer. " : "RightPlayer. " + "Shooting disabled.");
          isOver = true;
          setStatusText("Teammate abandoned. Waiting for teammate...");
          break;
        case Command.REPORT_DIRECTION:
          if (isLeft)
            rightCrossbow.setDirection(data[1]);
          else
            leftCrossbow.setDirection(data[1]);
          refresh();
          break;
        case Command.REPORT_FIRED:
          if (isLeft)
            rightCrossbow.fire(false);
          else
            leftCrossbow.fire(false);
          nbShotTeammate++;
          break;
        case Command.APPLE_DROPPED:
          long clockTime = TcpTools.toLong(data[1], data[2]);
          while (agent.getClockTime() < clockTime)
          {
          }
          createApple(new Location(data[3], data[4]));
          break;
        case Command.APPLE_HIT:
          nbHitTeammate++;
          Actor apple = getOneActor(Apple.class);
          Actor bolt = getOneActor(YourBolt.class);
          if (apple != null && bolt != null)
            addActor(new AppleHit(), apple.getLocation(), bolt.getDirection());
          if (apple != null)
            apple.removeSelf();
          if (bolt != null)
            bolt.removeSelf();
          refresh();
          break;
      }
    }
  }
  //
  private final static String VERSION = "1.01";
  private static final int nbBolt = 20;
  private String sessionID = "appleshooter124iun&%9";
  private final String bridgeName = "TellServer";
  private boolean isLeft;
  private String nickname = "Tell";
  private String myName;
  private String teammate;
  private TcpAgent agent;
  private Crossbow leftCrossbow;
  private Crossbow rightCrossbow;
  private int nbShotTeammate = 0;
  private int nbHitTeammate = 0;
  protected volatile static boolean isOver = true;
  protected volatile static boolean isQuiverEmpty = false;
  protected volatile static boolean isQuiverTeammateEmpty = false;
  private Actor isOverActor = new Actor("sprites/gameover.gif");

  public AppleShooter()
  {
    super(120, 100, 5, null, "sprites/forest.gif", false);
    String id = requestEntry("Session ID (>2 chars):");
    sessionID += id;
    agent = new TcpAgent(sessionID, bridgeName);
    addActor(isOverActor, new Location(50, 50));
    isOverActor.hide();
    setSimulationPeriod(20);
    addStatusBar(30);
    setTitle("AppleShooter Version " + VERSION);
    leftCrossbow = new Crossbow(agent, nickname + "(1)");
    addActor(leftCrossbow, new Location(15, 85));
    rightCrossbow = new Crossbow(agent, nickname);
    addActor(rightCrossbow, new Location(105, 85));
    rightCrossbow.setHorzMirror(true);
    show();
    agent.addTcpAgentListener(new MyTcpAgentAdapter());
    ArrayList<String> connectList = agent.connectToRelay(nickname);
    if (!agent.isBridgeConnected())
    {
      setStatusText("Connected, but game server in session ID " + id + " not found. Terminating now...");
      TcpTools.delay(3000);
      System.exit(0);
      return;
    }
    if (connectList.size() > 2)
    {
      setStatusText("Connected, but game already in process. Terminating now...");
      TcpTools.delay(3000);
      System.exit(0);
      return;
    }
    if (connectList.size() == 1)
    {
      setTitle("LeftPlayer. Shooting disabled.");
      setStatusText("Connected and waiting for teammate...");
      addKeyListener(leftCrossbow);
    }
    agent.sendSynchRequest();
    doRun();
    while (true)
    {
      Monitor.putSleep();
      TcpTools.delay(10000);
      initialize();
    }
  }

  private void initialize()
  {
    isOverActor.hide();
    isOver = false;
    isQuiverEmpty = false;
    isQuiverTeammateEmpty = false;
    Crossbow.nbShot = 0;
    Crossbow.nbHit = 0;
    nbShotTeammate = 0;
    nbHitTeammate = 0;
    leftCrossbow.setDirection(0);
    rightCrossbow.setDirection(0);
    refresh();
  }

  private void createApple(Location loc)
  {
    if (!isRunning())
      return;
    Apple apfel = new Apple();
    addActor(apfel, loc, 90);
    apfel.addCollisionActors((getActors(MyBolt.class)));
    apfel.addActorCollisionListener(leftCrossbow);
    apfel.addActorCollisionListener(rightCrossbow);
  }

  public void act()
  {
    if (isOver)
      return;
    setStatusText("My score: Shots " + Crossbow.nbShot
      + (isQuiverEmpty ? " (Quiver empty!)"
      : " (Bolts in Quiver: " + (nbBolt - Crossbow.nbShot) + ")")
      + " Hits: " + Crossbow.nbHit
      + " --- Teammate score: Shots: " + nbShotTeammate
      + (isQuiverTeammateEmpty ? " (Quiver empty!)"
      : " (Bolts in Quiver: " + (nbBolt - nbShotTeammate) + ")")
      + " Hits: " + nbHitTeammate);

    if (!isQuiverEmpty && Crossbow.nbShot >= nbBolt)
      isQuiverEmpty = true;
    if (!isQuiverTeammateEmpty && nbShotTeammate >= nbBolt)
      isQuiverTeammateEmpty = true;
    if (isQuiverEmpty && isQuiverTeammateEmpty)
    {
      isOver = true;
      isOverActor.show();
      int myHits = Crossbow.nbHit;
      int teammateHits = nbHitTeammate;
      String msg = "";
      if (myHits > teammateHits)
        msg = "I am the winner!";
      if (myHits < teammateHits)
        msg = "Teammate won!";
      if (myHits == teammateHits)
        msg = "Tie!";
      setStatusText("Game over. " + msg + " (My score: " + myHits
        + ". Teammate score: " + teammateHits
        + ")  Game will restart in 10 s. Please wait...");
      Monitor.wakeUp();
    }
  }

  private String requestEntry(String prompt)
  {
    String entry = "";
    while (entry.length() < 3)
    {
      entry = JOptionPane.showInputDialog(null, prompt, "");
      if (entry == null)
        System.exit(0);
    }
    return entry.trim();
  }

  public static void main(String[] args)
  {
    new AppleShooter();
  }
}
