// Crossbow.java

import ch.aplu.jgamegrid.*;
import java.awt.event.KeyEvent;
import ch.aplu.tcp.*;

class Crossbow extends Actor implements GGKeyListener
{
  protected static int nbShot = 0;
  protected static int nbHit = 0;
  private TcpAgent agent;
  private String teammate;
  private boolean isReleased = true;

  public Crossbow(TcpAgent agent, String teammate)
  {
    super(true, "sprites/crossbow.gif");
    this.agent = agent;
    this.teammate = teammate;
  }

  public boolean keyPressed(KeyEvent evt)
  {
    if (!isReleased)
      return true;
    isReleased = false;

    switch (evt.getKeyCode())
    {
      case KeyEvent.VK_UP:
        setDirection(getDirection() - 5);
        agent.sendCommand(teammate, Command.REPORT_DIRECTION,
          (int)getDirection());
        break;

      case KeyEvent.VK_DOWN:
        setDirection(getDirection() + 5);
        agent.sendCommand(teammate, Command.REPORT_DIRECTION,
          (int)getDirection());
        break;

      case KeyEvent.VK_SPACE:
        if (AppleShooter.isQuiverEmpty || AppleShooter.isOver)
          return false;
        new Thread()
        {
          public void run()
          {
            TcpTools.delay(100);  // Delay somewhat to compensate transfer time to teammate
            fire(true);
          }
        }.start();
        agent.sendCommand(teammate, Command.REPORT_FIRED);
        nbShot++;
        break;
    }
    return false;
  }

  public boolean keyReleased(KeyEvent evt)
  {
    isReleased = true;
    return true;
  }

  protected void fire(boolean local)
  {
    Bolt bolt;
    if (local)
    {
      bolt = new MyBolt();
      bolt.addCollisionActors((gameGrid.getActors(Apple.class)));
      bolt.addActorCollisionListener(this);
    }
    else
      bolt = new YourBolt();
    bolt.setSlowDown(2);
    if (isHorzMirror())
      gameGrid.addActor(bolt, getLocation(), 180 - getDirection());
    else
      gameGrid.addActor(bolt, getLocation(), getDirection());
  }

  public int collide(Actor a1, Actor a2)
  {
    agent.sendCommand(teammate, Command.APPLE_HIT);
    Actor apple;
    Actor bolt;
    if (a1 instanceof Apple)
    {
      apple = a1;
      bolt = a2;
    }
    else
    {
      apple = a2;
      bolt = a1;
    }
    gameGrid.addActor(new AppleHit(), apple.getLocation(), bolt.getDirection());
    a1.removeSelf();
    a2.removeSelf();
    nbHit++;
    return 0;
  }
}
