// AppleShooterServer.java

import ch.aplu.util.*;
import ch.aplu.tcp.*;
import java.util.*;
import javax.swing.JOptionPane;

public class AppleShooterServer extends Console
{
  private final static String VERSION = "1.01";
  private String sessionID = "appleshooter124iun&%9";
  private final String bridgeName = "TellServer";
  private TcpBridge bridge;
  private ArrayList<String> players = new ArrayList<String>();
  private boolean isInProgress = false;

  public AppleShooterServer()
  {
    String id = requestEntry("Session ID (>2 chars):");
    println("AppleShooterServer Version " + VERSION + " using session ID: " + id);
    sessionID += id;
    bridge = new TcpBridge(sessionID, bridgeName);

    bridge.addTcpBridgeListener(new TcpBridgeAdapter()
    {
      public void pipeRequest(String source, String destination, int[] data)
      {
        bridge.send(source, destination, data);
        showRequest(source, destination, data);
      }

      public void notifyAgentConnection(String agentName, boolean connected)
      {
        if (!(agentName.equals("Tell") ||
          agentName.equals("Tell(1)")))
          return;  // Deny other players
        println(agentName + (connected ? " connected" : " disconnected"));
        if (connected)
          players.add(agentName);
        else
          players.remove(agentName);
        if (players.size() == 2)
        {
          isInProgress = true;
          // Transmit player's name in parameter 'source'
          bridge.send(players.get(0), players.get(0), Command.GAME_START);
          bridge.send(players.get(1), players.get(1), Command.GAME_START);
          println("Game started.");
        }
        if (isInProgress && players.size() == 1)
        {
          isInProgress = false;
          bridge.send("", players.get(0), Command.TEAMMATE_QUIT);
          println("Teammate abandoned.");
        }
      }
    });
    print("Connecting to relay...");
    if (bridge.connectToRelay().size() > 1)
    {
      println("Connected.\nBut session ID in use.\nTerminating now...");
      TcpTools.delay(4000);
      System.exit(0);
    }

    print("done.\nRequesting clock time synchronisation...");
    int rc = bridge.sendSynchRequest();
    println("done. ");
    println("Travel time to relay and back: " + rc + " ms.");
    println("Waiting for players...");
    while (true)
    {
      int x = 40 + (int)(40 * Math.random());
      int y = 10;
      sendToAll(x, y);
      delay(3000);
    }
  }

  private void sendToAll(int x, int y)
  {
    int delay = 200;
    int[] time = TcpTools.toIntAry(bridge.getClockTime() + delay);
    for (String agent : players)
      bridge.send("", agent, Command.APPLE_DROPPED, time[0], time[1], x, y);
  }

  public void showRequest(String source, String destination, int[] data)
  {
    print("Pipe Request: " + source + "-->" + destination + " -- Data: ");
    for (int i = 0; i < data.length; i++)
      print(data[i] + " ");
    println();
  }

  private String requestEntry(String prompt)
  {
    String entry = "";
    while (entry.length() < 3)
    {
      entry = JOptionPane.showInputDialog(null, prompt, "");
      if (entry == null)
        System.exit(0);
    }
    return entry.trim();
  }

  public static void main(String[] args)
  {
    new AppleShooterServer();
  }
}
