// HitMePlayer.java

import ch.aplu.jgamegrid.*;
import ch.aplu.tcp.*;
import java.awt.*;

public class HitMePlayer extends GameGrid implements GGMouseListener
{
  private final String sessionID = "hitme7912302042";
  private final String bridgeName = "HitMeServer";
  private String agentName = "HitMePlayer";
  private String personalName;
  private TcpAgent agent = new TcpAgent(sessionID, bridgeName);
  private String hitter;
  private int nbHits = 0;
  private long dropTime;

  public HitMePlayer()
  {
    super(10, 10, 50, Color.green, false);
    addMouseListener(this, GGMouse.lPress);
    addStatusBar(30);
    playSound(this, GGSound.DUMMY);
    agent.addTcpAgentListener(new TcpAgentAdapter()
    {
      public void dataReceived(String source, int[] data)
      {
        switch (data[0])
        {
          case Command.INIT:
            removeAllActors();
            nbHits = 0;
            setStatusText("Game initialized");
            playSound(this, GGSound.NOTIFY);
            refresh();
            break;
          case Command.DROP:
            dropTime = TcpTools.toLong(data[1], data[2]);
            while (agent.getClockTime() < dropTime)
            {
            }
            Actor apple = new Actor("sprites/apple.gif");
            addActor(apple, new Location(data[3], data[4]));
            break;
          case Command.REMOVE:
            removeAllActors();
            refresh();
            hitter = TcpTools.intAryToString(data, 1);
            if (hitter.equals(personalName))
            {
              nbHits++;
              playSound(this, GGSound.PING);
            }
            break;
          case Command.TIME:
            int maxScore = data[1];
            long reactionTime = TcpTools.toLong(data[2], data[3]);
            setStatusText("Hitter: " +
              (hitter.equals(personalName)? "you" : hitter) + ". Reaction time: "
              + reactionTime + " ms. Your score: " + nbHits + " (Max: "
              + maxScore + ")");
            break;
        }
      }
    });
    show();
    personalName = agent.connectToRelay(agentName).get(0);
    setTitle("Personal name: " + personalName);
    setStatusText("Connection established. Personal name attributed: " + personalName);
    agent.sendSynchRequest();
  }

  public boolean mouseEvent(GGMouse mouse)
  {
    Location location = toLocationInGrid(mouse.getX(), mouse.getY());
    Actor a = getOneActorAt(location);
    if (a != null)
    {
      a.removeSelf();
      long reactionTime = agent.getClockTime() - dropTime;
      agent.sendCommand("", Command.HIT, TcpTools.toIntAry(reactionTime));
      refresh();
    }
    return true;
  }

  public static void main(String[] args)
  {
    new HitMePlayer();
  }
}
