// HitMeServer.java

import ch.aplu.util.*;
import ch.aplu.tcp.*;
import java.util.*;

public class HitMeServer extends Console
{
  private final String sessionID = "hitme7912302042";
  private final String bridgeName = "HitMeServer";
  private TcpBridge bridge = new TcpBridge(sessionID, bridgeName);
  private Hashtable<String, Integer> players = new Hashtable<String, Integer>();
  private boolean isRequestEnabled = true;
  private long dropTimeLong;

  public HitMeServer()
  {
    bridge.addTcpBridgeListener(new TcpBridgeAdapter()
    {
      public void pipeRequest(String source, String destination, int[] data)
      {
        if (!isRequestEnabled)
          return;
        isRequestEnabled = false; // Disable subsequent hits
        players.put(source, players.get(source) + 1);
        long reactionTime = TcpTools.toLong(data[1], data[2]);
        removeToAll(source, getMaxScore(), reactionTime);
        println("Got hit from " + source + ". Reaction time : "
          + reactionTime + " ms");
        Monitor.wakeUp();
      }

      public void notifyAgentConnection(String agentName, boolean connected)
      {
        println(agentName + (connected ? " connected" : " disconnected"));
        if (connected)
          players.put(agentName, 0);
        else
          players.remove(agentName);
        if (!players.isEmpty())
        {
          initToAll();
          Monitor.wakeUp();
        }
      }
    });
    print("Connecting to relay...");
    bridge.connectToRelay();
    println("done.\nWaiting for players...");
    bridge.sendSynchRequest();
    while (true)
    {
      Monitor.putSleep();
      TcpTools.delay((int)(1000 + 3000 * Math.random()));
      int x = (int)(10 * Math.random());
      int y = (int)(10 * Math.random());
      dropToAll(x, y);
    }
  }

  private void initToAll()
  {
    println("Game initialized");
    Enumeration e = players.keys();
    while (e.hasMoreElements())
    {
      String agent = (String)e.nextElement();
      bridge.sendCommand("", agent, Command.INIT);
      players.put(agent, 0);
    }
  }

  private void dropToAll(int x, int y)
  {
    println("Drop at (x, y) = (" + x + ", " + y + ")");
    dropTimeLong = bridge.getClockTime() + 500;  // 500 ms delay!
    int[] dropTime = TcpTools.toIntAry(dropTimeLong);
    Enumeration e = players.keys();
    while (e.hasMoreElements())
    {
      String agent = (String)e.nextElement();
      bridge.send("", agent, Command.DROP, dropTime[0], dropTime[1], x, y);
      isRequestEnabled = true;
    }
  }

  private void removeToAll(String hitter, int maxScore, long reactionTime)
  {
    Enumeration e = players.keys();
    while (e.hasMoreElements())
    {
      String agent = (String)e.nextElement();
      bridge.sendCommand("", agent, Command.REMOVE,
        TcpTools.stringToIntAry(hitter));
      int[] time = TcpTools.toIntAry(reactionTime);
      bridge.sendCommand("", agent, Command.TIME, maxScore, time[0], time[1]);
    }
  }

  private int getMaxScore()
  {
    int max = 0;
    Enumeration e = players.keys();
    while (e.hasMoreElements())
    {
      String agent = (String)e.nextElement();
      int score = players.get(agent);
      if (score > max)
        max = score;
    }
    return max;
  }

  public static void main(String[] args)
  {
    new HitMeServer();
  }
}
