// TicTacToeServer.java

import ch.aplu.tcp.*; 
import java.util.ArrayList;
import ch.aplu.util.Console;
import javax.swing.JOptionPane;

public class TicTacToeServer extends TcpBridge
{
  private static String VERSION = "1.3";
  private static String sessionID = "TicTacToe_67%&%2134";
  private static String id;
  private static final String bridgeName = "TicTacToeServer";
  // players.get(0) has mark 'X', players.get(1) has mark 'O'
  private ArrayList<String> players = new ArrayList<String>();
  private Console c;
  private char[][] gameState = new char[3][3];
  private int startRequest;
  private String[] startingGrid = new String[2];

  static
  {
    id = requestEntry("Game Room ID (>2 chars):");
    sessionID += id;
  }

  public TicTacToeServer()
  {
    super(sessionID, bridgeName);
    c = Console.init();
    c.setTitle("TicTacToeServer Version " + VERSION
      + " Using Game Room ID: " + id);
    addTcpBridgeListener(new TcpBridgeAdapter()
    {
      public void pipeRequest(String source, String destination, int[] data)
      {
        showPipeRequest(source, data);
        String teammate = source.contains("(1)")
          ? TcpTools.stripNickname(source) : (source + "(1)");

        switch (data[0])
        {
          case Command.MOVE:
            char value = data[3] == 0 ? 'O' : 'X';
            gameState[data[1]][data[2]] = value;
            send("", teammate, data);
            String[] playerState = getPlayerState();
            if (playerState == null)  // still in progress
            {
              if (isBoardFull())
              {
                c.println("Game over. It's a draw");
                send("", players.get(0), Command.TIE);
                send("", players.get(1), Command.TIE);
                // Exchange starting grid
                String temp = startingGrid[1];
                startingGrid[1] = startingGrid[0];
                startingGrid[0] = temp;
              }
            }
            else
            {
              c.println("Game over. Winner: " + playerState[0] + ". Loser: "
                + playerState[1]);
              send("", playerState[0], Command.WON);
              send("", playerState[1], Command.LOST);
              startingGrid[0] = playerState[1];  // Loser makes first move
              startingGrid[1] = playerState[0];
            }
            break;

          case Command.RESTART:
            startRequest++;
            if (startRequest == 2)
              initGame();
            break;
        }
      }

      public void notifyAgentConnection(String agentName, boolean connected)
      {
        c.println(agentName + (connected ? " connected" : " disconnected"));
        if (connected)
        {
          if (players.size() >= 2)
          {
            send("", agentName, Command.IN_PROGRESS);
            return;
          }

          players.add(agentName);

          if (players.size() == 1)
          {
            send("", players.get(0), Command.TEAMMATE_WAIT);
            c.println("Wait for teammate.");
          }

          if (players.size() == 2)
          {
            // First player logged in will start
            startingGrid[0] = players.get(0);
            startingGrid[1] = players.get(1);
            initGame();  
            c.println("Game started.");
          }
        }
        else  // disconnected
        {
          players.remove(agentName);
          if (players.size() == 1)
          {
            send("", players.get(0), Command.TEAMMATE_QUIT);
            c.println("Teammate abandoned.");
          }
        }
      }
    });

    c.print("Connecting to relay...");
    if (connectToRelay().size() > 1)
    {
      c.println("Connected.\nBut game room ID in use.\nTerminating now...");
      TcpTools.delay(4000);
      System.exit(0);
    }

    c.println("done.\nWaiting for players...");
  }

  private boolean isBoardFull()
  {
    int nb = 0;
    for (int i = 0; i < 3; i++)
      for (int k = 0; k < 3; k++)
        if (gameState[i][k] != ' ')
          nb++;
    return nb == 9;
  }

  private String[] getPlayerState()
  {
    // Convert gameState into string pattern
    StringBuffer pattern = new StringBuffer();
    // Horizontal
    for (int i = 0; i < 3; i++)
    {
      for (int k = 0; k < 3; k++)
      {
        pattern.append(gameState[i][k]);
      }
      pattern.append(',');  // Separator
    }
    // Vertical
    for (int k = 0; k < 3; k++)
    {
      for (int i = 0; i < 3; i++)
      {
        pattern.append(gameState[i][k]);
      }
      pattern.append(',');
    }
    // Diagonal
    for (int i = 0; i < 3; i++)
      pattern.append(gameState[i][i]);
    pattern.append(',');
    for (int i = 0; i < 3; i++)
      pattern.append(gameState[i][2 - i]);

    c.println("Pattern: " + pattern);
    String[] result = new String[2];
    if (pattern.toString().contains("XXX"))
    {
      result[0] = players.get(0);
      result[1] = players.get(1);
      return result;
    }
    if (pattern.toString().contains("OOO"))
    {
      result[0] = players.get(1);
      result[1] = players.get(0);
      return result;
    }
    return null;
  }

  private void initGame()
  {
    for (int i = 0; i < 3; i++)
      for (int k = 0; k < 3; k++)
        gameState[i][k] = ' ';
    startRequest = 0;
    send("", players.get(0), Command.MARK_X);
    send("", players.get(1), Command.MARK_O);
    // Send first to 'second' to make him ready to receive move requests
    send("", startingGrid[1], Command.START_SECOND);
    TcpTools.delay(2000);
    send("", startingGrid[0], Command.START_FIRST); 
  }

  private void showPipeRequest(String source, int[] data)
  {
    c.print("pipeRequest from: " + source);
    c.print(". Data: ");
    for (Integer k : data)
      c.print(k + " ");
    c.println();
  }

  private static String requestEntry(String prompt)
  {
    String entry = "";
    while (entry.length() < 3)
    {
      entry = JOptionPane.showInputDialog(null, prompt, "");
      if (entry == null)
        System.exit(0);
    }
    return entry.trim();
  }

  public static void main(String[] args)
  {
    new TicTacToeServer();
  }
}
